@@ -1,14 +1,10 @@
-# -*- coding: utf-8 -*-
-
 import attr
 import typing as ty
-import os, sys
 import pytest
-from pathlib import Path
-
 
 from ..task import ShellCommandTask
 from ..specs import ShellOutSpec, ShellSpec, SpecInfo, File
+from .utils import use_validator
 
 
 def test_shell_cmd_execargs_1():
@@ -479,9 +475,9 @@ def test_shell_cmd_inputs_format_1():
     )
 
     shelly = ShellCommandTask(
-        executable="executable", inpA="inpA", input_spec=my_input_spec
+        executable="executable", inpA="aaa", input_spec=my_input_spec
     )
-    assert shelly.cmdline == "executable -v inpA"
+    assert shelly.cmdline == "executable -v aaa"
 
 
 def test_shell_cmd_inputs_format_2():
@@ -1030,7 +1026,199 @@ def test_shell_cmd_inputs_template_6a():
     assert shelly.cmdline == "executable inpA"
 
 
-def test_shell_cmd_inputs_di(tmpdir):
+def test_shell_cmd_inputs_template_7(tmpdir):
+    """ additional inputs uses output_file_template with a suffix (no extension)
+    no keep_extension is used
+    """
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 2,
+                        "help_string": "outA",
+                        "argstr": "",
+                        "output_file_template": "{inpA}_out",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    inpA_file = tmpdir.join("a_file.txt")
+    inpA_file.write("content")
+    shelly = ShellCommandTask(
+        executable="executable", input_spec=my_input_spec, inpA=inpA_file
+    )
+
+    # outA should be formatted in a way that that .txt goes to the end
+    assert (
+        shelly.cmdline
+        == f"executable {tmpdir.join('a_file.txt')} {tmpdir.join('a_file_out.txt')}"
+    )
+
+
+def test_shell_cmd_inputs_template_7a(tmpdir):
+    """ additional inputs uses output_file_template with a suffix (no extension)
+        keep_extension is True (as default)
+    """
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 2,
+                        "help_string": "outA",
+                        "argstr": "",
+                        "keep_extension": True,
+                        "output_file_template": "{inpA}_out",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    inpA_file = tmpdir.join("a_file.txt")
+    inpA_file.write("content")
+    shelly = ShellCommandTask(
+        executable="executable", input_spec=my_input_spec, inpA=inpA_file
+    )
+
+    # outA should be formatted in a way that that .txt goes to the end
+    assert (
+        shelly.cmdline
+        == f"executable {tmpdir.join('a_file.txt')} {tmpdir.join('a_file_out.txt')}"
+    )
+
+
+def test_shell_cmd_inputs_template_7b(tmpdir):
+    """ additional inputs uses output_file_template with a suffix (no extension)
+    keep extension is False (so the extension is removed when creating the output)
+    """
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 2,
+                        "help_string": "outA",
+                        "argstr": "",
+                        "keep_extension": False,
+                        "output_file_template": "{inpA}_out",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    inpA_file = tmpdir.join("a_file.txt")
+    inpA_file.write("content")
+    shelly = ShellCommandTask(
+        executable="executable", input_spec=my_input_spec, inpA=inpA_file
+    )
+
+    # outA should be formatted in a way that that .txt goes to the end
+    assert (
+        shelly.cmdline
+        == f"executable {tmpdir.join('a_file.txt')} {tmpdir.join('a_file_out')}"
+    )
+
+
+def test_shell_cmd_inputs_template_8(tmpdir):
+    """additional inputs uses output_file_template with a suffix and an extension"""
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "inpA",
+                attr.ib(
+                    type=File,
+                    metadata={
+                        "position": 1,
+                        "help_string": "inpA",
+                        "argstr": "",
+                        "mandatory": True,
+                    },
+                ),
+            ),
+            (
+                "outA",
+                attr.ib(
+                    type=str,
+                    metadata={
+                        "position": 2,
+                        "help_string": "outA",
+                        "argstr": "",
+                        "output_file_template": "{inpA}_out.txt",
+                    },
+                ),
+            ),
+        ],
+        bases=(ShellSpec,),
+    )
+
+    inpA_file = tmpdir.join("a_file.t")
+    inpA_file.write("content")
+    shelly = ShellCommandTask(
+        executable="executable", input_spec=my_input_spec, inpA=inpA_file
+    )
+
+    # outA should be formatted in a way that inpA extension is removed and the template extension is used
+    assert (
+        shelly.cmdline
+        == f"executable {tmpdir.join('a_file.t')} {tmpdir.join('a_file_out.txt')}"
+    )
+
+
+def test_shell_cmd_inputs_di(tmpdir, use_validator):
     """ example from #279 """
     my_input_spec = SpecInfo(
         name="Input",
@@ -1086,20 +1274,6 @@ def test_shell_cmd_inputs_di(tmpdir):
                     },
                 ),
             ),
-            (
-                "noise_model",
-                attr.ib(
-                    type=int,
-                    metadata={
-                        "help_string": """
-            Rician/(Gaussian)
-            Employ a Rician or Gaussian noise model.
-            """,
-                        "allowed_values": ["Rician", "Gaussian"],
-                        "argstr": "-n",
-                    },
-                ),
-            ),
             (
                 "shrink_factor",
                 attr.ib(
@@ -1192,7 +1366,7 @@ def test_shell_cmd_inputs_di(tmpdir):
             (
                 "verbose",
                 attr.ib(
-                    type=bool,
+                    type=int,
                     default=0,
                     metadata={"help_string": "(0)/1. Verbose output. ", "argstr": "-v"},
                 ),
@@ -1222,7 +1396,7 @@ def test_shell_cmd_inputs_di(tmpdir):
         bases=(ShellSpec,),
     )
 
-    my_input_file = tmpdir.join("a_file")
+    my_input_file = tmpdir.join("a_file.ext")
     my_input_file.write("content")
 
     # no input provided
@@ -1239,7 +1413,7 @@ def test_shell_cmd_inputs_di(tmpdir):
     )
     assert (
         shelly.cmdline
-        == f"DenoiseImage -i {my_input_file} -s 1 -p 1 -r 2 -o [{my_input_file}_out]"
+        == f"DenoiseImage -i {tmpdir.join('a_file.ext')} -s 1 -p 1 -r 2 -o [{tmpdir.join('a_file_out.ext')}]"
     )
 
     # input file name, noiseImage is set to True, so template is used in the utput
@@ -1251,7 +1425,7 @@ def test_shell_cmd_inputs_di(tmpdir):
     )
     assert (
         shelly.cmdline
-        == f"DenoiseImage -i {my_input_file} -s 1 -p 1 -r 2 -o [{my_input_file}_out, {my_input_file}_noise]"
+        == f"DenoiseImage -i {tmpdir.join('a_file.ext')} -s 1 -p 1 -r 2 -o [{tmpdir.join('a_file_out.ext')}, {tmpdir.join('a_file_noise.ext')}]"
     )
 
     # input file name and help_short
@@ -1263,7 +1437,7 @@ def test_shell_cmd_inputs_di(tmpdir):
     )
     assert (
         shelly.cmdline
-        == f"DenoiseImage -i {my_input_file} -s 1 -p 1 -r 2 -h -o [{my_input_file}_out]"
+        == f"DenoiseImage -i {tmpdir.join('a_file.ext')} -s 1 -p 1 -r 2 -h -o [{tmpdir.join('a_file_out.ext')}]"
     )
 
     assert shelly.output_names == [
@@ -1273,3 +1447,25 @@ def test_shell_cmd_inputs_di(tmpdir):
         "correctedImage",
         "noiseImage",
     ]
+
+    # adding image_dimensionality that has allowed_values [2, 3, 4]
+    shelly = ShellCommandTask(
+        executable="DenoiseImage",
+        inputImageFilename=my_input_file,
+        input_spec=my_input_spec,
+        image_dimensionality=2,
+    )
+    assert (
+        shelly.cmdline
+        == f"DenoiseImage -d 2 -i {tmpdir.join('a_file.ext')} -s 1 -p 1 -r 2 -o [{tmpdir.join('a_file_out.ext')}]"
+    )
+
+    # adding image_dimensionality that has allowed_values [2, 3, 4] and providing 5 - exception should be raised
+    with pytest.raises(ValueError) as excinfo:
+        shelly = ShellCommandTask(
+            executable="DenoiseImage",
+            inputImageFilename=my_input_file,
+            input_spec=my_input_spec,
+            image_dimensionality=5,
+        )
+    assert "value of image_dimensionality" in str(excinfo.value)
